@@ -37,8 +37,6 @@
 
 #define NAME_TO_CLAIM "org.freedesktop.Accounts"
 
-static GMainLoop *loop;
-
 static gboolean
 ensure_directory (const char  *path,
                   GError     **error)
@@ -59,36 +57,28 @@ on_bus_acquired (GDBusConnection  *connection,
                  const gchar      *name,
                  gpointer          user_data)
 {
+        GMainLoop *loop = user_data;
         Daemon *daemon;
-        GError *local_error = NULL;
-        GError **error = &local_error;
+        g_autoptr(GError) error = NULL;
 
-        if (!ensure_directory (ICONDIR, error)) {
-                goto out;
-        }
-
-        if (!ensure_directory (USERDIR, error)) {
-                goto out;
+        if (!ensure_directory (ICONDIR, &error) ||
+            !ensure_directory (USERDIR, &error)) {
+                g_printerr ("%s\n", error->message);
+                g_main_loop_quit (loop);
+                return;
         }
 
         daemon = daemon_new ();
         if (daemon == NULL) {
-                g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                             "Failed to initialize daemon");
-                goto out;
+                g_printerr ("Failed to initialize daemon\n");
+                g_main_loop_quit (loop);
+                return;
         }
 
         openlog ("accounts-daemon", LOG_PID, LOG_DAEMON);
         syslog (LOG_INFO, "started daemon version %s", VERSION);
         closelog ();
         openlog ("accounts-daemon", 0, LOG_AUTHPRIV);
-
- out:
-        if (local_error != NULL) {
-                g_printerr ("%s\n", local_error->message);
-                g_clear_error (&local_error);
-                g_main_loop_quit (loop);
-        }
 }
 
 static void
@@ -96,6 +86,8 @@ on_name_lost (GDBusConnection  *connection,
               const gchar      *name,
               gpointer          user_data)
 {
+        GMainLoop *loop = user_data;
+
         g_debug ("got NameLost, exiting");
         g_main_loop_quit (loop);
 }
@@ -108,7 +100,7 @@ on_log_debug (const gchar *log_domain,
               const gchar *message,
               gpointer user_data)
 {
-        GString *string;
+        g_autoptr(GString) string = NULL;
         const gchar *progname;
         int ret G_GNUC_UNUSED;
 
@@ -121,8 +113,6 @@ on_log_debug (const gchar *log_domain,
                                 message ? message : "(NULL) message");
 
         ret = write (1, string->str, string->len);
-
-        g_string_free (string, TRUE);
 }
 
 static void
@@ -141,17 +131,19 @@ log_handler (const gchar   *domain,
 static gboolean
 on_signal_quit (gpointer data)
 {
-        g_main_loop_quit (data);
+        GMainLoop *loop = data;
+
+        g_main_loop_quit (loop);
         return FALSE;
 }
 
 int
 main (int argc, char *argv[])
 {
-        GError *error;
-        gint ret;
+        g_autoptr(GMainLoop) loop = NULL;
+        g_autoptr(GError) error = NULL;
         GBusNameOwnerFlags flags;
-        GOptionContext *context;
+        g_autoptr(GOptionContext) context = NULL;
         static gboolean replace;
         static gboolean show_version;
         static GOptionEntry entries[] = {
@@ -162,9 +154,6 @@ main (int argc, char *argv[])
                 { NULL }
         };
 
-        ret = 1;
-        error = NULL;
-
         setlocale (LC_ALL, "");
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
@@ -174,25 +163,21 @@ main (int argc, char *argv[])
 
         if (!g_setenv ("GIO_USE_VFS", "local", TRUE)) {
                 g_warning ("Couldn't set GIO_USE_GVFS");
-                goto out;
+                return EXIT_FAILURE;
         }
 
         context = g_option_context_new ("");
         g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
         g_option_context_set_summary (context, _("Provides D-Bus interfaces for querying and manipulating\nuser account information."));
         g_option_context_add_main_entries (context, entries, NULL);
-        error = NULL;
         if (!g_option_context_parse (context, &argc, &argv, &error)) {
                 g_warning ("%s", error->message);
-                g_error_free (error);
-                goto out;
+                return EXIT_FAILURE;
         }
-        g_option_context_free (context);
 
         if (show_version) {
                 g_print ("accounts-daemon " VERSION "\n");
-                ret = 0;
-                goto out;
+                return EXIT_SUCCESS;
         }
 
         /* If --debug, then print debug messages even when no G_MESSAGES_DEBUG */
@@ -200,6 +185,8 @@ main (int argc, char *argv[])
                 g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, on_log_debug, NULL);
         g_log_set_default_handler (log_handler, NULL);
 
+        loop = g_main_loop_new (NULL, FALSE);
+
         flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT;
         if (replace)
                 flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
@@ -209,11 +196,9 @@ main (int argc, char *argv[])
                         on_bus_acquired,
                         NULL,
                         on_name_lost,
-                        NULL,
+                        loop,
                         NULL);
 
-        loop = g_main_loop_new (NULL, FALSE);
-
         g_unix_signal_add (SIGINT, on_signal_quit, loop);
         g_unix_signal_add (SIGTERM, on_signal_quit, loop);
 
@@ -221,11 +206,7 @@ main (int argc, char *argv[])
         g_main_loop_run (loop);
 
         g_debug ("exiting");
-        g_main_loop_unref (loop);
 
-        ret = 0;
-
- out:
-        return ret;
+        return EXIT_SUCCESS;
 }
 
